package org.apache.bcel.generic;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import org.apache.bcel.classfile.Constant;
import org.apache.bcel.classfile.ConstantCP;
import org.apache.bcel.classfile.ConstantClass;
import org.apache.bcel.classfile.ConstantDouble;
import org.apache.bcel.classfile.ConstantDynamic;
import org.apache.bcel.classfile.ConstantFieldref;
import org.apache.bcel.classfile.ConstantFloat;
import org.apache.bcel.classfile.ConstantInteger;
import org.apache.bcel.classfile.ConstantInterfaceMethodref;
import org.apache.bcel.classfile.ConstantInvokeDynamic;
import org.apache.bcel.classfile.ConstantLong;
import org.apache.bcel.classfile.ConstantMethodref;
import org.apache.bcel.classfile.ConstantNameAndType;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.ConstantString;
import org.apache.bcel.classfile.ConstantUtf8;
import org.apache.bcel.classfile.Utility;

/* loaded from: org_apache_bcel_generic.jar:org/apache/bcel/generic/ConstantPoolGen.class */
public class ConstantPoolGen {
    private static final int DEFAULT_BUFFER_SIZE = 256;
    private static final String METHODREF_DELIM = ":";
    private static final String IMETHODREF_DELIM = "#";
    private static final String FIELDREF_DELIM = "&";
    private static final String NAT_DELIM = "%";

    @Deprecated
    protected int size;

    @Deprecated
    protected Constant[] constants;

    @Deprecated
    protected int index;
    private final Map<String, Integer> stringTable;
    private final Map<String, Integer> classTable;
    private final Map<String, Integer> utf8Table;
    private final Map<String, Integer> natTable;
    private final Map<String, Integer> cpTable;

    public ConstantPoolGen() {
        this.index = 1;
        this.stringTable = new HashMap();
        this.classTable = new HashMap();
        this.utf8Table = new HashMap();
        this.natTable = new HashMap();
        this.cpTable = new HashMap();
        this.size = DEFAULT_BUFFER_SIZE;
        this.constants = new Constant[this.size];
    }

    public ConstantPoolGen(Constant[] cs) {
        String className;
        this.index = 1;
        this.stringTable = new HashMap();
        this.classTable = new HashMap();
        this.utf8Table = new HashMap();
        this.natTable = new HashMap();
        this.cpTable = new HashMap();
        StringBuilder sb = new StringBuilder(DEFAULT_BUFFER_SIZE);
        this.size = Math.min(Math.max(DEFAULT_BUFFER_SIZE, cs.length + 64), 65536);
        this.constants = new Constant[this.size];
        System.arraycopy(cs, 0, this.constants, 0, cs.length);
        if (cs.length > 0) {
            this.index = cs.length;
        }
        for (int i = 1; i < this.index; i++) {
            ConstantString constantString = this.constants[i];
            if (constantString instanceof ConstantString) {
                ConstantString s = constantString;
                ConstantUtf8 u8 = this.constants[s.getStringIndex()];
                String key = u8.getBytes();
                if (!this.stringTable.containsKey(key)) {
                    this.stringTable.put(key, Integer.valueOf(i));
                }
            } else if (constantString instanceof ConstantClass) {
                ConstantClass s2 = (ConstantClass) constantString;
                ConstantUtf8 u82 = this.constants[s2.getNameIndex()];
                String key2 = u82.getBytes();
                if (!this.classTable.containsKey(key2)) {
                    this.classTable.put(key2, Integer.valueOf(i));
                }
            } else if (constantString instanceof ConstantNameAndType) {
                ConstantNameAndType n = (ConstantNameAndType) constantString;
                ConstantUtf8 u83 = this.constants[n.getNameIndex()];
                ConstantUtf8 u8_2 = this.constants[n.getSignatureIndex()];
                sb.append(u83.getBytes());
                sb.append(NAT_DELIM);
                sb.append(u8_2.getBytes());
                String key3 = sb.toString();
                sb.delete(0, sb.length());
                if (!this.natTable.containsKey(key3)) {
                    this.natTable.put(key3, Integer.valueOf(i));
                }
            } else if (constantString instanceof ConstantUtf8) {
                ConstantUtf8 u = (ConstantUtf8) constantString;
                String key4 = u.getBytes();
                if (!this.utf8Table.containsKey(key4)) {
                    this.utf8Table.put(key4, Integer.valueOf(i));
                }
            } else if (constantString instanceof ConstantCP) {
                ConstantInvokeDynamic constantInvokeDynamic = (ConstantCP) constantString;
                if (constantString instanceof ConstantInvokeDynamic) {
                    className = Integer.toString(constantInvokeDynamic.getBootstrapMethodAttrIndex());
                } else if (constantString instanceof ConstantDynamic) {
                    className = Integer.toString(((ConstantDynamic) constantInvokeDynamic).getBootstrapMethodAttrIndex());
                } else {
                    ConstantClass clazz = this.constants[constantInvokeDynamic.getClassIndex()];
                    ConstantUtf8 u84 = this.constants[clazz.getNameIndex()];
                    className = Utility.pathToPackage(u84.getBytes());
                }
                ConstantNameAndType n2 = this.constants[constantInvokeDynamic.getNameAndTypeIndex()];
                ConstantUtf8 u85 = this.constants[n2.getNameIndex()];
                String methodName = u85.getBytes();
                ConstantUtf8 u86 = this.constants[n2.getSignatureIndex()];
                String signature = u86.getBytes();
                String delim = METHODREF_DELIM;
                if (constantString instanceof ConstantInterfaceMethodref) {
                    delim = IMETHODREF_DELIM;
                } else if (constantString instanceof ConstantFieldref) {
                    delim = FIELDREF_DELIM;
                }
                sb.append(className);
                sb.append(delim);
                sb.append(methodName);
                sb.append(delim);
                sb.append(signature);
                String key5 = sb.toString();
                sb.delete(0, sb.length());
                if (!this.cpTable.containsKey(key5)) {
                    this.cpTable.put(key5, Integer.valueOf(i));
                }
            }
        }
    }

    public ConstantPoolGen(ConstantPool cp) {
        this(cp.getConstantPool());
    }

    public int addArrayClass(ArrayType type) {
        return addClass_(type.getSignature());
    }

    public int addClass(ObjectType type) {
        return addClass(type.getClassName());
    }

    public int addClass(String str) {
        return addClass_(str.replace('.', '/'));
    }

    private int addClass_(String clazz) {
        int cpRet = lookupClass(clazz);
        if (cpRet != -1) {
            return cpRet;
        }
        adjustSize();
        Constant constantClass = new ConstantClass(addUtf8(clazz));
        int ret = this.index;
        Constant[] constantArr = this.constants;
        int i = this.index;
        this.index = i + 1;
        constantArr[i] = constantClass;
        return computeIfAbsent(this.classTable, clazz, ret);
    }

    public int addConstant(Constant c, ConstantPoolGen cp) {
        ConstantUtf8[] constantPool = cp.getConstantPool().getConstantPool();
        switch (c.getTag()) {
            case 1:
                return addUtf8(((ConstantUtf8) c).getBytes());
            case 2:
            default:
                throw new IllegalArgumentException("Unknown constant type " + c);
            case 3:
                return addInteger(((ConstantInteger) c).getBytes());
            case 4:
                return addFloat(((ConstantFloat) c).getBytes());
            case 5:
                return addLong(((ConstantLong) c).getBytes());
            case 6:
                return addDouble(((ConstantDouble) c).getBytes());
            case 7:
                ConstantClass s = (ConstantClass) c;
                ConstantUtf8 u8 = constantPool[s.getNameIndex()];
                return addClass(u8.getBytes());
            case 8:
                ConstantString s2 = (ConstantString) c;
                ConstantUtf8 u82 = constantPool[s2.getStringIndex()];
                return addString(u82.getBytes());
            case 9:
            case 10:
            case 11:
                ConstantCP m = (ConstantCP) c;
                ConstantClass clazz = (ConstantClass) constantPool[m.getClassIndex()];
                ConstantNameAndType n = (ConstantNameAndType) constantPool[m.getNameAndTypeIndex()];
                ConstantUtf8 u83 = constantPool[clazz.getNameIndex()];
                String className = Utility.pathToPackage(u83.getBytes());
                ConstantUtf8 u84 = constantPool[n.getNameIndex()];
                String name = u84.getBytes();
                ConstantUtf8 u85 = constantPool[n.getSignatureIndex()];
                String signature = u85.getBytes();
                switch (c.getTag()) {
                    case 9:
                        return addFieldref(className, name, signature);
                    case 10:
                        return addMethodref(className, name, signature);
                    case 11:
                        return addInterfaceMethodref(className, name, signature);
                    default:
                        throw new IllegalArgumentException("Unknown constant type " + c);
                }
            case 12:
                ConstantNameAndType n2 = (ConstantNameAndType) c;
                ConstantUtf8 u86 = constantPool[n2.getNameIndex()];
                ConstantUtf8 u8_2 = constantPool[n2.getSignatureIndex()];
                return addNameAndType(u86.getBytes(), u8_2.getBytes());
        }
    }

    public int addDouble(double n) {
        int ret = lookupDouble(n);
        if (ret != -1) {
            return ret;
        }
        adjustSize();
        int ret2 = this.index;
        this.constants[this.index] = new ConstantDouble(n);
        this.index += 2;
        return ret2;
    }

    public int addFieldref(String className, String fieldName, String signature) {
        int cpRet = lookupFieldref(className, fieldName, signature);
        if (cpRet != -1) {
            return cpRet;
        }
        adjustSize();
        int classIndex = addClass(className);
        int nameAndTypeIndex = addNameAndType(fieldName, signature);
        int ret = this.index;
        Constant[] constantArr = this.constants;
        int i = this.index;
        this.index = i + 1;
        constantArr[i] = new ConstantFieldref(classIndex, nameAndTypeIndex);
        return computeIfAbsent(this.cpTable, className + FIELDREF_DELIM + fieldName + FIELDREF_DELIM + signature, ret);
    }

    public int addFloat(float n) {
        int ret = lookupFloat(n);
        if (ret != -1) {
            return ret;
        }
        adjustSize();
        int ret2 = this.index;
        Constant[] constantArr = this.constants;
        int i = this.index;
        this.index = i + 1;
        constantArr[i] = new ConstantFloat(n);
        return ret2;
    }

    public int addInteger(int n) {
        int ret = lookupInteger(n);
        if (ret != -1) {
            return ret;
        }
        adjustSize();
        int ret2 = this.index;
        Constant[] constantArr = this.constants;
        int i = this.index;
        this.index = i + 1;
        constantArr[i] = new ConstantInteger(n);
        return ret2;
    }

    public int addInterfaceMethodref(MethodGen method) {
        return addInterfaceMethodref(method.getClassName(), method.getName(), method.getSignature());
    }

    public int addInterfaceMethodref(String className, String methodName, String signature) {
        int cpRet = lookupInterfaceMethodref(className, methodName, signature);
        if (cpRet != -1) {
            return cpRet;
        }
        adjustSize();
        int classIndex = addClass(className);
        int nameAndTypeIndex = addNameAndType(methodName, signature);
        int ret = this.index;
        Constant[] constantArr = this.constants;
        int i = this.index;
        this.index = i + 1;
        constantArr[i] = new ConstantInterfaceMethodref(classIndex, nameAndTypeIndex);
        return computeIfAbsent(this.cpTable, className + IMETHODREF_DELIM + methodName + IMETHODREF_DELIM + signature, ret);
    }

    public int addLong(long n) {
        int ret = lookupLong(n);
        if (ret != -1) {
            return ret;
        }
        adjustSize();
        int ret2 = this.index;
        this.constants[this.index] = new ConstantLong(n);
        this.index += 2;
        return ret2;
    }

    public int addMethodref(MethodGen method) {
        return addMethodref(method.getClassName(), method.getName(), method.getSignature());
    }

    public int addMethodref(String className, String methodName, String signature) {
        int cpRet = lookupMethodref(className, methodName, signature);
        if (cpRet != -1) {
            return cpRet;
        }
        adjustSize();
        int nameAndTypeIndex = addNameAndType(methodName, signature);
        int classIndex = addClass(className);
        int ret = this.index;
        Constant[] constantArr = this.constants;
        int i = this.index;
        this.index = i + 1;
        constantArr[i] = new ConstantMethodref(classIndex, nameAndTypeIndex);
        return computeIfAbsent(this.cpTable, className + METHODREF_DELIM + methodName + METHODREF_DELIM + signature, ret);
    }

    public int addNameAndType(String name, String signature) {
        int ret = lookupNameAndType(name, signature);
        if (ret != -1) {
            return ret;
        }
        adjustSize();
        int nameIndex = addUtf8(name);
        int signatureIndex = addUtf8(signature);
        int ret2 = this.index;
        Constant[] constantArr = this.constants;
        int i = this.index;
        this.index = i + 1;
        constantArr[i] = new ConstantNameAndType(nameIndex, signatureIndex);
        return computeIfAbsent(this.natTable, name + NAT_DELIM + signature, ret2);
    }

    public int addString(String str) {
        int ret = lookupString(str);
        if (ret != -1) {
            return ret;
        }
        int utf8 = addUtf8(str);
        adjustSize();
        Constant constantString = new ConstantString(utf8);
        int ret2 = this.index;
        Constant[] constantArr = this.constants;
        int i = this.index;
        this.index = i + 1;
        constantArr[i] = constantString;
        return computeIfAbsent(this.stringTable, str, ret2);
    }

    public int addUtf8(String n) {
        int ret = lookupUtf8(n);
        if (ret != -1) {
            return ret;
        }
        adjustSize();
        int ret2 = this.index;
        Constant[] constantArr = this.constants;
        int i = this.index;
        this.index = i + 1;
        constantArr[i] = new ConstantUtf8(n);
        return computeIfAbsent(this.utf8Table, n, ret2);
    }

    protected void adjustSize() {
        if (this.index + 3 >= 65536) {
            throw new IllegalStateException("The number of constants " + (this.index + 3) + " is over the size of the constant pool: 65535");
        }
        if (this.index + 3 >= this.size) {
            Constant[] cs = this.constants;
            this.size *= 2;
            this.size = Math.min(this.size, 65536);
            this.constants = new Constant[this.size];
            System.arraycopy(cs, 0, this.constants, 0, this.index);
        }
    }

    private int computeIfAbsent(Map<String, Integer> map, String key, int value) {
        return map.computeIfAbsent(key, k -> {
            return Integer.valueOf(value);
        }).intValue();
    }

    public Constant getConstant(int i) {
        return this.constants[i];
    }

    public ConstantPool getConstantPool() {
        return new ConstantPool(this.constants);
    }

    public ConstantPool getFinalConstantPool() {
        return new ConstantPool((Constant[]) Arrays.copyOf(this.constants, this.index));
    }

    private int getIndex(Map<String, Integer> map, String key) {
        return toIndex(map.get(key));
    }

    public int getSize() {
        return this.index;
    }

    public int lookupClass(String str) {
        return getIndex(this.classTable, str.replace('.', '/'));
    }

    public int lookupDouble(double n) {
        long bits = Double.doubleToLongBits(n);
        for (int i = 1; i < this.index; i++) {
            if (this.constants[i] instanceof ConstantDouble) {
                ConstantDouble c = this.constants[i];
                if (Double.doubleToLongBits(c.getBytes()) == bits) {
                    return i;
                }
            }
        }
        return -1;
    }

    public int lookupFieldref(String className, String fieldName, String signature) {
        return getIndex(this.cpTable, className + FIELDREF_DELIM + fieldName + FIELDREF_DELIM + signature);
    }

    public int lookupFloat(float n) {
        int bits = Float.floatToIntBits(n);
        for (int i = 1; i < this.index; i++) {
            if (this.constants[i] instanceof ConstantFloat) {
                ConstantFloat c = this.constants[i];
                if (Float.floatToIntBits(c.getBytes()) == bits) {
                    return i;
                }
            }
        }
        return -1;
    }

    public int lookupInteger(int n) {
        for (int i = 1; i < this.index; i++) {
            if (this.constants[i] instanceof ConstantInteger) {
                ConstantInteger c = this.constants[i];
                if (c.getBytes() == n) {
                    return i;
                }
            }
        }
        return -1;
    }

    public int lookupInterfaceMethodref(MethodGen method) {
        return lookupInterfaceMethodref(method.getClassName(), method.getName(), method.getSignature());
    }

    public int lookupInterfaceMethodref(String className, String methodName, String signature) {
        return getIndex(this.cpTable, className + IMETHODREF_DELIM + methodName + IMETHODREF_DELIM + signature);
    }

    public int lookupLong(long n) {
        for (int i = 1; i < this.index; i++) {
            if (this.constants[i] instanceof ConstantLong) {
                ConstantLong c = this.constants[i];
                if (c.getBytes() == n) {
                    return i;
                }
            }
        }
        return -1;
    }

    public int lookupMethodref(MethodGen method) {
        return lookupMethodref(method.getClassName(), method.getName(), method.getSignature());
    }

    public int lookupMethodref(String className, String methodName, String signature) {
        return getIndex(this.cpTable, className + METHODREF_DELIM + methodName + METHODREF_DELIM + signature);
    }

    public int lookupNameAndType(String name, String signature) {
        return getIndex(this.natTable, name + NAT_DELIM + signature);
    }

    public int lookupString(String str) {
        return getIndex(this.stringTable, str);
    }

    public int lookupUtf8(String n) {
        return getIndex(this.utf8Table, n);
    }

    public void setConstant(int i, Constant c) {
        this.constants[i] = c;
    }

    private int toIndex(Integer index) {
        if (index != null) {
            return index.intValue();
        }
        return -1;
    }

    public String toString() {
        StringBuilder buf = new StringBuilder();
        for (int i = 1; i < this.index; i++) {
            buf.append(i).append(")").append(this.constants[i]).append("\n");
        }
        return buf.toString();
    }
}
